var __defProp = Object.defineProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", {value: true});
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, {get: all[name], enumerable: true});
};

// src/index.ts
__markAsModule(exports);
__export(exports, {
  Container: () => Container,
  createContainer: () => createContainer,
  injected: () => injected,
  tag: () => tag,
  tagged: () => tagged,
  token: () => token
});

// src/globals/entitiesRegistry.ts
var entitiesRegistry = new Map();

// src/globals/injectsRegistry.ts
var injectsRegistry = new Map();

// src/globals/tagsRegistry.ts
var tagsRegistry = new Map();

// src/container/bindings/Binding.ts
var Type;
(function(Type2) {
  Type2[Type2["Constant"] = 0] = "Constant";
  Type2[Type2["Entity"] = 1] = "Entity";
  Type2[Type2["Factory"] = 2] = "Factory";
})(Type || (Type = {}));
var Scope;
(function(Scope2) {
  Scope2[Scope2["Container"] = 0] = "Container";
  Scope2[Scope2["Global"] = 1] = "Global";
  Scope2[Scope2["Resolution"] = 2] = "Resolution";
  Scope2[Scope2["Singleton"] = 3] = "Singleton";
  Scope2[Scope2["Transient"] = 4] = "Transient";
})(Scope || (Scope = {}));

// src/container/bindings/ConstantBinding.ts
var ConstantBinding = class {
  constructor(value) {
    this.value = value;
    this.type = Type.Constant;
    this.scope = Scope.Transient;
  }
};

// src/container/bindings/EntityBinding.ts
var EntityContainerScopedBinding = class {
  constructor(value, isConstructor) {
    this.value = value;
    this.isConstructor = isConstructor;
    this.type = Type.Entity;
    this.scope = Scope.Container;
    this.cache = new WeakMap();
  }
};
var EntityGlobalScopedBinding = class {
  constructor(value, isConstructor) {
    this.value = value;
    this.isConstructor = isConstructor;
    this.type = Type.Entity;
    this.scope = Scope.Global;
  }
};
var EntityResolutionScopedBinding = class {
  constructor(value, isConstructor) {
    this.value = value;
    this.isConstructor = isConstructor;
    this.type = Type.Entity;
    this.scope = Scope.Resolution;
  }
};
var EntitySingletonScopedBinding = class {
  constructor(value, isConstructor) {
    this.value = value;
    this.isConstructor = isConstructor;
    this.type = Type.Entity;
    this.scope = Scope.Singleton;
  }
};
var EntityTransientScopedBinding = class {
  constructor(value, isConstructor) {
    this.value = value;
    this.isConstructor = isConstructor;
    this.type = Type.Entity;
    this.scope = Scope.Transient;
  }
};
var isEntityBinding = (binding) => binding.type === Type.Entity;
var isEntityConstructorBinding = (binding) => binding.isConstructor;
var isEntityContainerScopedBinding = (binding) => binding.scope === Scope.Container;
var isEntityGlobalScopedBinding = (binding) => binding.scope === Scope.Global;
var isEntityResolutionScopedBinding = (binding) => binding.scope === Scope.Resolution;
var isEntitySingletonScopedBinding = (binding) => binding.scope === Scope.Singleton;

// src/container/bindings/FactoryBinding.ts
var FactoryBinding = class {
  constructor(value) {
    this.value = value;
    this.type = Type.Factory;
    this.scope = Scope.Transient;
  }
};
var isFactoryBinding = (binding) => binding.type === Type.Factory;
var isFactoryConstructorBinding = (binding) => binding.value.isConstructor;

// src/container/syntax/ScopeSyntax.ts
var ScopeSyntax = class {
  constructor(bindingsVault, value, isConstructor, token2, condition) {
    this.bindingsVault = bindingsVault;
    this.value = value;
    this.isConstructor = isConstructor;
    this.token = token2;
    this.condition = condition;
    if (process.env.NODE_ENV !== "production") {
      this.warningTimeout = setTimeout(() => {
        console.warn(`Warning: did you forget to set a scope for '${this.token.__symbol.description}' token binding? Call 'inTransientScope()', 'inSingletonScope()', 'inContainerScope()' or 'inResolutionScope()'.`);
      });
    }
  }
  inContainerScope() {
    this.set(new EntityContainerScopedBinding(this.value, this.isConstructor));
  }
  inGlobalScope() {
    this.set(new EntityGlobalScopedBinding(this.value, this.isConstructor));
  }
  inResolutionScope() {
    this.set(new EntityResolutionScopedBinding(this.value, this.isConstructor));
  }
  inSingletonScope() {
    this.set(new EntitySingletonScopedBinding(this.value, this.isConstructor));
  }
  inTransientScope() {
    this.set(new EntityTransientScopedBinding(this.value, this.isConstructor));
  }
  set(binding) {
    if (process.env.NODE_ENV !== "production")
      clearTimeout(this.warningTimeout);
    this.bindingsVault.set(binding, this.token, this.condition);
  }
};

// src/container/syntax/TypeSyntax.ts
var TypeSyntax = class {
  constructor(bindingsVault, token2, condition) {
    this.bindingsVault = bindingsVault;
    this.token = token2;
    this.condition = condition;
  }
  toInstance(ctor) {
    return new ScopeSyntax(this.bindingsVault, ctor, true, this.token, this.condition);
  }
  toCall(func) {
    return new ScopeSyntax(this.bindingsVault, func, false, this.token, this.condition);
  }
  toConstant(value) {
    this.bindingsVault.set(new ConstantBinding(value), this.token, this.condition);
  }
  toFactory(ctor, initializer) {
    this.bindingsVault.set(new FactoryBinding({creator: ctor, initializer, isConstructor: true}), this.token, this.condition);
  }
  toCreator(func, initializer) {
    this.bindingsVault.set(new FactoryBinding({creator: func, initializer, isConstructor: false}), this.token, this.condition);
  }
};

// src/container/syntax/BindSyntax.ts
var BindSyntax = class {
  constructor(bindingsVault, condition) {
    this.bindingsVault = bindingsVault;
    this.condition = condition;
  }
  bind(token2) {
    return new TypeSyntax(this.bindingsVault, token2, this.condition);
  }
};

// src/pointers/tag.ts
var tag = (description) => Symbol(description);

// src/pointers/token.ts
var token = (description) => {
  const symbol = Symbol(description);
  return {
    __type: null,
    __symbol: symbol,
    __isOptional: false,
    optional: {
      __type: null,
      __symbol: symbol,
      __isOptional: true
    }
  };
};

// src/container/BindingsVault.ts
var _BindingsVault = class {
  constructor() {
    this.map = new Map();
  }
  set(binding, token2, condition = _BindingsVault.notag) {
    const current = this.map.get(token2.__symbol);
    if (current === void 0) {
      this.map.set(token2.__symbol, new Map().set(condition, binding));
    } else {
      current.set(condition, binding);
    }
  }
  get(token2, conditions = [], target) {
    const bindings = this.map.get(token2.__symbol);
    if (bindings === void 0)
      return void 0;
    if (target !== void 0) {
      const targetBinding = bindings.get(target);
      if (targetBinding)
        return targetBinding;
    }
    if (process.env.NODE_ENV !== "production" && conditions.reduce((acc, condition) => bindings.has(condition) ? acc + 1 : acc, 0) > 1) {
      const conditionsDisplayString = conditions.map((condition) => typeof condition === "function" ? condition.name : `tag(${condition.description})`).join(", ");
      console.warn(`Warning: When resolving a binding by '${token2.__symbol.description}' token with [${conditionsDisplayString}] conditions, more than one binding was found. In this case, Brandi resolves the binding by the first tag assigned by 'tagged(target, ...tags)' function or, if you explicitly passed conditions through 'Container.get(token, conditions)' method, by the first resolved condition. Try to avoid such implicit logic.`);
    }
    for (let i = 0, len = conditions.length; i < len; i += 1) {
      const binding = bindings.get(conditions[i]);
      if (binding)
        return binding;
    }
    return bindings.get(_BindingsVault.notag);
  }
  clone() {
    const newBindingsVault = new _BindingsVault();
    this.map.forEach((value, key) => {
      newBindingsVault.map.set(key, new Map(value));
    });
    return newBindingsVault;
  }
};
var BindingsVault = _BindingsVault;
BindingsVault.notag = tag("notag");

// src/container/Container.ts
var Container = class {
  constructor(parent) {
    this.parent = parent;
    this.vault = new BindingsVault();
    this.snapshot = null;
  }
  clone() {
    const newContainer = new Container(this.parent);
    newContainer.vault = this.vault.clone();
    return newContainer;
  }
  capture() {
    this.snapshot = this.vault.clone();
  }
  restore() {
    if (this.snapshot !== null) {
      this.vault = this.snapshot.clone();
    } else if (process.env.NODE_ENV !== "production") {
      console.error("Error: It looks like a trying to restore a non-captured container state. Did you forget to call 'capture()' method?");
    }
  }
  bind(token2) {
    return new BindSyntax(this.vault).bind(token2);
  }
  when(condition) {
    return new BindSyntax(this.vault, condition);
  }
  get(token2, conditions) {
    return this.getSingle(token2, conditions);
  }
  getSingle(token2, conditions, target, context = new Map()) {
    const binding = this.resolveBinding(token2, conditions, target);
    if (binding === null)
      return void 0;
    return this.resolveValue(binding, context);
  }
  getMultiple(tokens, context, conditions, target) {
    return tokens.map((token2) => this.getSingle(token2, conditions, target, context));
  }
  resolveBinding(token2, conditions, target) {
    const binding = this.vault.get(token2, conditions, target);
    if (binding !== void 0)
      return binding;
    if (this.parent !== void 0)
      return this.parent.resolveBinding(token2);
    if (token2.__isOptional)
      return null;
    throw new Error(`No matching bindings found for '${token2.__symbol.description}' token.`);
  }
  resolveValue(binding, context) {
    if (isEntityBinding(binding)) {
      if (isEntitySingletonScopedBinding(binding)) {
        return this.resolveEntityCache(binding, context, () => binding.cache, (entity) => {
          binding.cache = entity;
        });
      }
      if (isEntityContainerScopedBinding(binding)) {
        return this.resolveEntityCache(binding, context, () => binding.cache.get(this), (entity) => {
          binding.cache.set(this, entity);
        });
      }
      if (isEntityResolutionScopedBinding(binding)) {
        return this.resolveEntityCache(binding, context, () => context.get(binding), (entity) => {
          context.set(binding, entity);
        });
      }
      if (isEntityGlobalScopedBinding(binding)) {
        return this.resolveEntityCache(binding, context, () => entitiesRegistry.get(binding.value), (entity) => {
          entitiesRegistry.set(binding.value, entity);
        });
      }
      return this.resolveEntity(binding, context);
    }
    if (isFactoryBinding(binding)) {
      return (...args) => {
        const entity = isFactoryConstructorBinding(binding) ? this.construct(binding.value.creator, context) : this.call(binding.value.creator, context);
        if (binding.value.initializer)
          binding.value.initializer(entity, ...args);
        return entity;
      };
    }
    return binding.value;
  }
  resolveEntityCache(binding, context, getCache, setCache) {
    const cache = getCache();
    if (cache !== void 0)
      return cache;
    const entity = this.resolveEntity(binding, context);
    setCache(entity);
    return entity;
  }
  resolveEntity(binding, context) {
    if (isEntityConstructorBinding(binding))
      return this.construct(binding.value, context);
    return this.call(binding.value, context);
  }
  resolveParameters(target, context) {
    const injects = injectsRegistry.get(target);
    if (injects === void 0) {
      if (target.length === 0)
        return [];
      throw new Error(`Missing required 'injected' registration of '${target.name}'`);
    }
    const tags = tagsRegistry.get(target);
    return this.getMultiple(injects, context, tags, target);
  }
  call(func, context) {
    return func(...this.resolveParameters(func, context));
  }
  construct(Ctor, context) {
    return new Ctor(...this.resolveParameters(Ctor, context));
  }
};

// src/container/createContainer.ts
var createContainer = (...args) => new Container(...args);

// src/registrators/injected.ts
var injected = (target, ...tokens) => {
  injectsRegistry.set(target, tokens);
  return target;
};

// src/registrators/tagged.ts
var tagged = (target, ...tags) => {
  tagsRegistry.set(target, tags);
  return target;
};
//# sourceMappingURL=brandi.js.map
